print GETDATE()

----------------------------------------------------------------------
-- Create a user that would be used for read only query connections --
----------------------------------------------------------------------

--- Create user
CREATE LOGIN ro   
    WITH PASSWORD = 'hp.123456'; 

--- Grant privs
EXEC sp_addsrvrolemember ro,'dbcreator';
EXEC sp_addsrvrolemember ro, 'sysadmin';
EXEC sp_addsrvrolemember ro, 'setupadmin';
EXEC sp_addsrvrolemember ro,'diskadmin';
EXEC sp_addsrvrolemember ro,'processadmin';
EXEC sp_addsrvrolemember ro,'serveradmin';
EXEC sp_addsrvrolemember ro,'securityadmin';  


---------------------------------------------------------
-- Configure/Reconfigure for read only query workloads --
---------------------------------------------------------

--- Define  RESOURCE POOL,  AFFINITY SCHEDULER for RO
IF EXISTS (SELECT name FROM sys.resource_governor_resource_pools WHERE name  =  'pRO' )
    ALTER RESOURCE POOL pRO WITH (AFFINITY SCHEDULER = (0 to 27, 56 to 83), MAX_MEMORY_PERCENT = 85)
ELSE
    CREATE RESOURCE POOL pRO WITH (AFFINITY SCHEDULER = (0 to 27, 56 to 83), MAX_MEMORY_PERCENT = 85)
GO
ALTER RESOURCE GOVERNOR RECONFIGURE;
GO

--- Create or Alter gRO Workload Group
IF EXISTS (SELECT name FROM sys.resource_governor_workload_groups WHERE name  =  'gRO' )
       ALTER WORKLOAD GROUP gRO WITH (request_max_memory_grant_percent = 45) USING pRO
ELSE
       CREATE WORKLOAD GROUP gRO WITH (request_max_memory_grant_percent = 45) USING pRO
GO
ALTER RESOURCE GOVERNOR RECONFIGURE;
GO

--------------------------------------------
-- Configure/Reconfigure for RF workloads --
--------------------------------------------

--- Define  RESOURCE POOL,  AFFINITY SCHEDULER for RF
IF EXISTS (SELECT name FROM sys.resource_governor_resource_pools WHERE name  =  'pRF' )
    ALTER RESOURCE POOL pRF WITH (AFFINITY SCHEDULER = (28 to 55, 84 to 111),MAX_MEMORY_PERCENT = 15)
ELSE
    CREATE RESOURCE POOL pRF WITH (AFFINITY SCHEDULER = (28 to 55, 84 to 111),MAX_MEMORY_PERCENT = 15)
GO
ALTER RESOURCE GOVERNOR RECONFIGURE;
GO

ALTER RESOURCE GOVERNOR RECONFIGURE;
GO
--- Create or Alter gRF Workload Group
IF EXISTS (SELECT name FROM sys.resource_governor_workload_groups WHERE name  =  'gRF' )
       ALTER WORKLOAD GROUP gRF WITH (request_max_memory_grant_percent = 80) USING pRF
ELSE
       CREATE WORKLOAD GROUP gRF WITH (request_max_memory_grant_percent = 80) USING pRF
GO
ALTER RESOURCE GOVERNOR RECONFIGURE;
GO

--------------------------------------------
-- Create and set the classifier function --
--------------------------------------------

--Delete existing TPCHClassifier
USE master
GO

IF EXISTS (
     SELECT  *     FROM    dbo.sysobjects     WHERE   id = OBJECT_ID(N'dbo.TPCHClassifier')
	 ) BEGIN
		ALTER RESOURCE GOVERNOR WITH (CLASSIFIER_FUNCTION=null); 
		ALTER RESOURCE GOVERNOR RECONFIGURE;
		DROP Function dbo.TPCHClassifier;
	   END

--Create new TPCHClassifier 
USE master
GO
SET ANSI_NULLS ON;
GO
SET QUOTED_IDENTIFIER ON;
GO
CREATE FUNCTION dbo.TPCHClassifier() RETURNS sysname
    WITH SCHEMABINDING
AS
BEGIN
    DECLARE @WorkloadGroup AS SYSNAME
    IF (SUSER_NAME () = 'ro')
        SET @WorkloadGroup = 'gRO'
    IF (SUSER_NAME () = 'sa')
        SET @WorkloadGroup = 'gRF'
RETURN @WorkloadGroup
END
GO

ALTER RESOURCE GOVERNOR WITH (CLASSIFIER_FUNCTION=dbo.TPCHClassifier);  
 
ALTER RESOURCE GOVERNOR RECONFIGURE
GO

print GETDATE()
